﻿using System;
using System.IO;
using System.Drawing;

// Thumbnail cache decoder
// Copyright (c) 2010 Dmitry Brant, all rights reserved
// http://dmitrybrant.com
// me@dmitrybrant.com

// This source code is free for personal use.
// For commercial use, please contact the developer to
// purchase a license.

// This Software is provided "as-is," without any express or 
// implied warranty, without even the implied warranty of 
// merchantability and fitness for a particular purpose. In 
// no event shall the Author be held liable for any, direct or 
// indirect, damages arising from the use of the Software.

namespace ThumbCacheViewer
{
    public class ThumbCache
    {

        public ThumbCache(string fileName)
        {
            tempBytes = new byte[65536];
            stream = new FileStream(fileName, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);

            stream.Read(tempBytes, 0, 24);
            if ((tempBytes[0] != 'C') || (tempBytes[1] != 'M') || (tempBytes[2] != 'M') || (tempBytes[3] != 'M'))
                throw new ApplicationException("This is not a valid ThumbCache file.");

            var reader = new BinaryReader(stream);

            UInt32 headerSize = BitConverter.ToUInt32(tempBytes, 12);
            UInt32 offsetLastEntry = BitConverter.ToUInt32(tempBytes, 16);
            numImages = (int)BitConverter.ToUInt32(tempBytes, 20);
            stream.Seek(headerSize, SeekOrigin.Begin);

            entryOffsets = new UInt32[numImages];
            UInt32 totalEntrySize;

            for (int i = 0; i < numImages; i++)
            {
                entryOffsets[i] = (UInt32)stream.Position;

                stream.Read(tempBytes, 0, 8);
                if ((tempBytes[0] != 'C') || (tempBytes[1] != 'M') || (tempBytes[2] != 'M') || (tempBytes[3] != 'M'))
                    break;

                totalEntrySize = BitConverter.ToUInt32(tempBytes, 4);
                stream.Seek(totalEntrySize - 8, SeekOrigin.Current);
            }


        }


        ~ThumbCache()
        {
            stream.Close();
        }


        private Stream stream;

        private byte[] tempBytes;
        private UInt32[] entryOffsets;



        private int numImages = 0;
        public int ImageCount { get { return numImages; } }


        public Image GetImage(int imageIndex)
        {
            Image img = null;
            if (numImages == 0) return img;
            if ((imageIndex < 0) || (imageIndex >= numImages)) return img;

            stream.Seek(entryOffsets[imageIndex], SeekOrigin.Begin);
            stream.Read(tempBytes, 0, 28);
            if ((tempBytes[0] != 'C') || (tempBytes[1] != 'M') || (tempBytes[2] != 'M') || (tempBytes[3] != 'M'))
                return img;

            UInt32 totalEntrySize = BitConverter.ToUInt32(tempBytes, 4);
            UInt32 dataSize = BitConverter.ToUInt32(tempBytes, 24);
            if (dataSize == 0) return img;

            stream.Seek(entryOffsets[imageIndex] + totalEntrySize - dataSize, SeekOrigin.Begin);

            if (dataSize > tempBytes.Length)
                tempBytes = new byte[dataSize];

            stream.Read(tempBytes, 0, (int)dataSize);
            var mstream = new MemoryStream(tempBytes);
            img = new Bitmap(mstream);
            return img;
        }





        public void Read(string fileName)
        {
            using (var f = new FileStream(fileName, FileMode.Open, FileAccess.Read, FileShare.Read))
            {
                Read(f);
            }
        }

        public void Read(Stream stream)
        {


        }

    }

}
